﻿using System;
using System.ComponentModel;
using System.Data;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using ObentoApp.ObentoDataSetTableAdapters;
using System.Windows.Media.Imaging;
using System.IO;
using Microsoft.Win32;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;

namespace ObentoApp
{
    /// <summary>
    /// ContetObentoEntry.xaml の相互作用ロジック
    /// </summary>
    public partial class ContetObentoEntry : UserControl
    {
        private MainWindow main;            // メイン・ウィンドウ
        private CollectionViewSource myCVS; // コレクションビューソース

        public ContetObentoEntry()
        {
            InitializeComponent();

            // MainWindowを取得して、各リボン・ボタンのイベント・ハンドラを追加
            main = (MainWindow)Application.Current.MainWindow;
            main.ButtonAddObentoEntry.Click += new RoutedEventHandler(ButtonAddObentoEntry_Click);
            main.ButtonFirstObentoEntry.Click += new RoutedEventHandler(ButtonFirstObentoEntry_Click);
            main.ButtonPrevObentoEntry.Click += new RoutedEventHandler(ButtonPrevObentoEntry_Click);
            main.ButtonNextObentoEntry.Click += new RoutedEventHandler(ButtonNextObentoEntry_Click);
            main.ButtonLastObentoEntry.Click += new RoutedEventHandler(ButtonLastObentoEntry_Click);
            main.ButtonDelObentoEntry.Click += new RoutedEventHandler(ButtonDelObentoEntry_Click);
        }

        #region データの初期ロードと、［登録］ボタンのClickイベント・ハンドラ

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            // デザイン時にはデータは読み込まれません
            if (!DesignerProperties.GetIsInDesignMode(this))
            {
                // XAMLファイル側で作成されたコレクションビューソースを取得
                myCVS = (CollectionViewSource)this.Resources["obentoViewSource"];
                // コレクションビューソース内の新規行に移動
                myCVS.View.MoveCurrentTo(null);

                ChangeButtonState();
            }
        }

        private void ButtonAdd_Click(object sender, RoutedEventArgs e)
        {
            // お弁当名を取得
            string name = nameTextBox.Text.Trim();
            if (String.IsNullOrEmpty(name))
            {
                MessageBox.Show("お弁当名を入力してください。");
                return;
            }
            // 説明を取得
            string desc = descriptionTextBox.Text.Trim();
            if (String.IsNullOrEmpty(desc))
            {
                MessageBox.Show("説明を入力してください。");
                return;
            }

            // 写真を取得
            byte[] photo = null;
            BitmapImage photoImage = ImagePreview.Source as BitmapImage;
            if (photoImage != null)
            {
                MemoryStream stream = photoImage.StreamSource as MemoryStream;
                photo = stream.ToArray(); // ストリームからバイト配列を取得
            }

            bool addMode = (myCVS.View.CurrentPosition == -1);
            if (addMode)
            {   // 新規登録モード
                // データセット内のObentoデータテーブルに新規行を追加
                main.ObentoDS.Obento.AddObentoRow(name, desc, photo);
            }
            else
            {   // 編集モード
                ObentoDataSet.ObentoRow row = ((DataRowView)myCVS.View.CurrentItem).Row as ObentoDataSet.ObentoRow;
                row.Name = name;
                row.Description = desc;
                row.Photo = photo;
            }
            //  Obentoテーブルアダプタで、Obentoデータをデータベースに書き込む
            try
            {
                main.ObentoTA.Update(main.ObentoDS.Obento);
            }
            catch (DBConcurrencyException de)
            {
                MessageBox.Show("登録に失敗しました。\n" + de.Message);
                return;
            }
            catch (Exception ex)
            {
                MessageBox.Show("エラーが発生しました。\n" + ex.Message);
                return;
            }

            // プレビュー画像を削除する
            photo = null;
            ImagePreview.Source = null;

            // コレクションビューを更新
            myCVS.View.Refresh();

            if (addMode)
            {   // 新規登録モード
                // コレクションビューソース内の新規行に移動
                myCVS.View.MoveCurrentTo(null);
            }
            else
            {   // 編集モード
                if (myCVS.View.CurrentPosition < ((CollectionView)myCVS.View).Count - 1)
                {
                    myCVS.View.MoveCurrentToNext();
                }
                else
                {
                    myCVS.View.MoveCurrentTo(null);
                }
            }
            ChangeButtonState();
        }

        #endregion

        #region リボン・ボタンのClickイベント・ハンドラ

        private void ButtonAddObentoEntry_Click(object sender, RoutedEventArgs e)
        {
            myCVS.View.MoveCurrentTo(null);
            ChangeButtonState();
        }

        private void ButtonFirstObentoEntry_Click(object sender, RoutedEventArgs e)
        {
            myCVS.View.MoveCurrentToFirst();
            ChangeButtonState();
        }

        private void ButtonPrevObentoEntry_Click(object sender, RoutedEventArgs e)
        {
            if (myCVS.View.CurrentPosition == -1)
            {
                // 新規登録モードから編集モードへ変更された場合
                myCVS.View.MoveCurrentToLast();
            }
            else if (myCVS.View.CurrentPosition > 0)
            {
                myCVS.View.MoveCurrentToPrevious();
            }
            ChangeButtonState();
        }

        private void ButtonNextObentoEntry_Click(object sender, RoutedEventArgs e)
        {
            if (myCVS.View.CurrentPosition == -1)
            {
                // 新規登録モードから編集モードへ変更された場合
                myCVS.View.MoveCurrentToLast();
            }
            else if (myCVS.View.CurrentPosition < ((CollectionView)myCVS.View).Count - 1)
            {
                myCVS.View.MoveCurrentToNext();
            }
            ChangeButtonState();
        }

        private void ButtonLastObentoEntry_Click(object sender, RoutedEventArgs e)
        {
            myCVS.View.MoveCurrentToLast();
            ChangeButtonState();
        }

        private void ButtonDelObentoEntry_Click(object sender, RoutedEventArgs e)
        {
            e.Handled = true;

            if (MessageBox.Show("現在のデータを1件、削除します。OKでしょうか？",
                "確認", MessageBoxButton.OKCancel, MessageBoxImage.Question,
                MessageBoxResult.Cancel) == MessageBoxResult.Cancel)
                return;

            if (myCVS.View.CurrentPosition == -1)
            {
                return;
            }

            // 現在のObentoデータ行を取得
            ((DataRowView)myCVS.View.CurrentItem).Row.Delete();

            // Obentoテーブルアダプタで、データをデータベースに書き込む
            try
            {
                // 先にOrderテーブル内の関連データを削除
                DataRow[] rows = main.ObentoDS.Order.Select("", "", DataViewRowState.Deleted);
                if (rows.Length > 0)
                {
                    main.OrderTA.Update(rows);
                }
                // 続いてObentoテーブル内の該当データを削除
                main.ObentoTA.Update(main.ObentoDS.Obento);
            }
            catch (DBConcurrencyException de)
            {
                MessageBox.Show("削除に失敗しました。すでに誰かによって削除されたようです。\n" + de.Message);
                return;
            }
            catch (Exception ex)
            {
                MessageBox.Show("エラーが発生しました。\n" + ex.Message);
                return;
            }

            // コレクションビューを更新
            myCVS.View.Refresh();

            ChangeButtonState();
        }

        private void ChangeButtonState()
        {
            // 新規登録モードの場合
            ButtonAdd.Content = "登録";
            if (myCVS.View.CurrentPosition == -1)
            {
                this.Background = this.FindResource("BackgroundAddMode") as System.Windows.Media.Brush;
                bool existObento = (((CollectionView)myCVS.View).Count > 0);
                main.ButtonFirstObentoEntry.IsEnabled = existObento;
                main.ButtonPrevObentoEntry.IsEnabled = existObento;
                main.ButtonNextObentoEntry.IsEnabled = existObento;
                main.ButtonLastObentoEntry.IsEnabled = existObento;
                main.ButtonAddObentoEntry.IsEnabled = false;
                main.ButtonDelObentoEntry.IsEnabled = false;
                return;
            }

            // 編集モードの場合
            this.Background = this.FindResource("BackgroundEditMode") as System.Windows.Media.Brush;
            ButtonAdd.Content = "更新";
            main.ButtonAddObentoEntry.IsEnabled = true;
            main.ButtonDelObentoEntry.IsEnabled = true;
            bool existPrev = (myCVS.View.CurrentPosition > 0);
            main.ButtonFirstObentoEntry.IsEnabled = existPrev;
            main.ButtonPrevObentoEntry.IsEnabled = existPrev;
            int count = ((CollectionView)myCVS.View).Count;
            bool existNext = (myCVS.View.CurrentPosition < count - 1);
            main.ButtonNextObentoEntry.IsEnabled = existNext;
            main.ButtonLastObentoEntry.IsEnabled = existNext;
        }

        #endregion

        private void ButtonRefPic_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".png";
            dlg.Filter = "画像ファイル(*.png;*.bmp;*.jpg;*.gif)|*.png;*.bmp;*.jpg;*.gif";
            bool? result = dlg.ShowDialog();
            if (result == true)
            {
                BitmapImage imageSrc = new BitmapImage();
                MemoryStream stream = new MemoryStream();
                Bitmap bitmap = MakeSquare(new Bitmap(dlg.FileName), 150);
                bitmap.Save(stream, ImageFormat.Png);
                bitmap.Dispose();

                imageSrc.BeginInit();
                imageSrc.StreamSource = stream;
                imageSrc.EndInit();

                ImagePreview.Source = imageSrc;
            }
        }
        
        public Bitmap MakeSquare(Bitmap imageOriginal, int toMaxWidth)
        {
            int x;
            int y;
            int fromWidth;
            int fromHeight;
            int fromMaxWidth;
            int xSquare;
            int ySquare;
            int widthSquare;
            int heightSquare;
            if (imageOriginal.Width > imageOriginal.Height)
            {
                if (imageOriginal.Height > toMaxWidth)
                {
                    // （1）toMaxWidthが「490」の場合：「700×500」は横幅700を500にカット（正方形）
                    x = (imageOriginal.Width - imageOriginal.Height) / 2;
                    y = 0;
                    fromWidth = imageOriginal.Height;
                    fromHeight = imageOriginal.Height;
                    // 実際の描画サイズ
                    xSquare = 0;
                    ySquare = 0;
                    widthSquare = toMaxWidth;
                    heightSquare = toMaxWidth;
                }
                else if (imageOriginal.Width >= toMaxWidth)
                {
                    // （2）toMaxWidthが「490」の場合：「700×350」は横幅700を490にカット（横長方形）
                    x = (imageOriginal.Width - toMaxWidth) / 2;
                    y = 0;
                    fromWidth = toMaxWidth;
                    fromHeight = imageOriginal.Height;
                    // 実際の描画サイズ
                    xSquare = 0;
                    ySquare = (toMaxWidth - fromHeight) / 2;
                    widthSquare = toMaxWidth;
                    heightSquare = fromHeight;
                }
                else
                {
                    // （3）toMaxWidthが「490」の場合：「450×300」は何もカットしない（横長方形）
                    x = 0;
                    y = 0;
                    fromWidth = imageOriginal.Width;
                    fromHeight = imageOriginal.Height;
                    // 実際の描画サイズ
                    xSquare = (toMaxWidth - fromWidth) / 2;
                    ySquare = (toMaxWidth - fromHeight) / 2;
                    widthSquare = fromWidth;
                    heightSquare = fromHeight;
                }
                fromMaxWidth = fromWidth;
            }
            else if (imageOriginal.Width < imageOriginal.Height)
            {
                if (imageOriginal.Width > toMaxWidth)
                {
                    // （4）toMaxWidthが「490」の場合：「500×700」は高さ700を500にカット（正方形）
                    x = 0;
                    y = (imageOriginal.Height - imageOriginal.Width) / 2;
                    fromWidth = imageOriginal.Width;
                    fromHeight = imageOriginal.Width;
                    // 実際の描画サイズ
                    xSquare = 0;
                    ySquare = 0;
                    widthSquare = toMaxWidth;
                    heightSquare = toMaxWidth;
                }
                else if (imageOriginal.Height >= toMaxWidth)
                {
                    // （5）toMaxWidthが「490」の場合：「350×700」は高さ700を490にカット（縦長方形）
                    x = 0;
                    y = (imageOriginal.Height - toMaxWidth) / 2;
                    fromWidth = imageOriginal.Width;
                    fromHeight = toMaxWidth;
                    // 実際の描画サイズ
                    xSquare = (toMaxWidth - fromWidth) / 2;
                    ySquare = 0;
                    widthSquare = fromWidth;
                    heightSquare = toMaxWidth;
                }
                else
                {
                    // （6）toMaxWidthが「490」の場合：「350×450」は何もカットしない（縦長方形）
                    x = 0;
                    y = 0;
                    fromWidth = imageOriginal.Width;
                    fromHeight = imageOriginal.Height;
                    // 実際の描画サイズ
                    xSquare = (toMaxWidth - fromWidth) / 2;
                    ySquare = (toMaxWidth - fromHeight) / 2;
                    widthSquare = fromWidth;
                    heightSquare = fromHeight;
                }
                fromMaxWidth = fromHeight;
            }
            else
            {
                // （7）toMaxWidthが「490」の場合：「350×350」は何もカットしない（正方形）
                // （8）toMaxWidthが「490」の場合：「700×700」は何もカットしない（正方形）
                x = 0;
                y = 0;
                fromWidth = imageOriginal.Width;
                fromHeight = imageOriginal.Height;
                fromMaxWidth = fromWidth;
                // 実際の描画サイズ
                xSquare = 0;
                ySquare = 0;
                widthSquare = toMaxWidth;
                heightSquare = toMaxWidth;
            }
            System.Drawing.Rectangle rect = new System.Drawing.Rectangle(x, y, fromWidth, fromHeight);
            Bitmap imageSquare = imageOriginal.Clone(rect, imageOriginal.PixelFormat);

            // オリジナル画像からサムネイル画像 (bothWidth x bothWidth) への縮小描画
            Bitmap imageThumbnail = new Bitmap(toMaxWidth, toMaxWidth);
            using (Graphics gcs = Graphics.FromImage(imageThumbnail))
            {
                gcs.FillRectangle(new SolidBrush(System.Drawing.Color.Transparent), 0, 0, toMaxWidth, toMaxWidth);
                gcs.InterpolationMode = InterpolationMode.HighQualityBicubic;
                gcs.SmoothingMode = SmoothingMode.AntiAlias;
                gcs.PixelOffsetMode = PixelOffsetMode.HighQuality;
                gcs.DrawImage(imageSquare, xSquare, ySquare, widthSquare, heightSquare);
            }

            imageSquare.Dispose();
            imageOriginal.Dispose();

            return imageThumbnail;
        }

    }
}
