﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using ObentoApp.ObentoDataSetTableAdapters;
using System.ComponentModel;
using System;
using System.Data;
using System.Windows.Media;
using System.Data.SqlClient;

namespace ObentoApp
{
    /// <summary>
    /// ContetObentoOrder.xaml の相互作用ロジック
    /// </summary>
    public partial class ContetObentoOrder : UserControl
    {
        private MainWindow main;             // メイン・ウィンドウ
        private ObentoDataSet myDS;          // データセット
        private OrderTableAdapter orderTA;   // Orderテーブルアダプタ
        private ObentoTableAdapter obentoTA; // Obentoテーブルアダプタ
        private UserTableAdapter userTA;     // Userテーブルアダプタ
        private CollectionViewSource myCVS;  // コレクションビューソース

        public ContetObentoOrder()
        {
            InitializeComponent();

            // MainWindowを取得して、各リボン・ボタンのイベント・ハンドラを追加
            main = (MainWindow)Application.Current.MainWindow;
            main.ButtonAddOrderEntry.Click += new RoutedEventHandler(ButtonAddOrderEntry_Click);
            main.ButtonFirstOrderEntry.Click += new RoutedEventHandler(ButtonFirstOrderEntry_Click);
            main.ButtonPrevOrderEntry.Click += new RoutedEventHandler(ButtonPrevOrderEntry_Click);
            main.ButtonNextOrderEntry.Click += new RoutedEventHandler(ButtonNextOrderEntry_Click);
            main.ButtonLastOrderEntry.Click += new RoutedEventHandler(ButtonLastOrderEntry_Click);
            main.ButtonDelOrderEntry.Click += new RoutedEventHandler(ButtonDelOrderEntry_Click);
            // ユーザー選択のイベント・ハンドラを追加
            main.UserSelection.SelectionChanged += new SelectionChangedEventHandler(UserSelection_SelectionChanged);
        }

        #region データの初期ロードと、［登録］ボタンのClickイベント・ハンドラ

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            // デザイン時にはデータは読み込まれません
            if (!DesignerProperties.GetIsInDesignMode(this))
            {
                // XAMLファイル側で作成されたデータセットを取得
                myDS = ((ObentoDataSet)(this.FindResource("obentoDataSet")));
                //myDS.EnforceConstraints = false;

                // Obentoテーブルアダプタで、データセットにObentoデータを読み込む
                obentoTA = new ObentoTableAdapter();
                try
                {
                    obentoTA.Fill(myDS.Obento);
                }
                catch (Exception ex)
                {
                    MessageBox.Show("お弁当データの読み込みでエラーが発生しました。\n" + ex.Message);
                    return;
                }
                // XAMLファイル側に配置したコンボボックスにObentoデータを設定
                ObentoComboBox.ItemsSource = obentoTA.GetData();
                ObentoComboBox.DisplayMemberPath = "Name";

                // Userテーブルアダプタで、データセットにUserデータを読み込む
                userTA = new UserTableAdapter();
                try
                {
                    userTA.Fill(myDS.User);
                }
                catch (Exception ex)
                {
                    MessageBox.Show("ユーザー・データの読み込みでエラーが発生しました。\n" + ex.Message);
                    return;
                }

                // Orderテーブルアダプタを生成
                orderTA = new OrderTableAdapter();
                // Orderテーブルアダプタで、データセットにObentoデータを読み込む
                if (main.SelectedUser != null)
                {
                    try
                    {
                        orderTA.FillBy(myDS.Order, main.SelectedUser.ID);                        
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show("注文データの読み込みでエラーが発生しました。\n" + ex.Message);
                        return;
                    }
                }

                // XAMLファイル側で作成されたコレクションビューソースを取得
                myCVS = (CollectionViewSource)this.Resources["obentoViewSource"];
                //myCVS = new CollectionViewSource(); //myCVS.Source = myDS.Order;
                // コレクションビューソース内の新規行に移動
                InitViewSource();
            }
            ChangeButtonState();
        }

        private void ButtonOrder_Click(object sender, RoutedEventArgs e)
        {
            // ユーザーが選択されているか確認
            if (main.SelectedUser == null)
            {
                MessageBox.Show("ユーザーを選択してください。");
                return;
            }

            // 注文日を取得
            DateTime? orderDate = OrderDatePicker.SelectedDate;
            if (orderDate.HasValue == false)
            {
                MessageBox.Show("注文日を入力してください。");
                return;
            }

            // お弁当を取得
            object comboSel = ObentoComboBox.SelectedValue;
            if (comboSel == null)
            {
                MessageBox.Show("お弁当を選択してください。");
                return;
            }
            ObentoDataSet.ObentoRow obento = (ObentoDataSet.ObentoRow)((DataRowView)comboSel).Row;

            bool addMode = (myCVS.View.CurrentPosition == -1);
            if (addMode)  // 新規登録モード
            {
                // データセット内のOrderデータテーブルに新規行を追加
                ObentoDataSet.UserRow ur = myDS.User.FindByID(main.SelectedUser.ID);
                if (ur == null)
                {
                    MessageBox.Show("ユーザーの特定に失敗しました。");
                    return;
                }
                myDS.Order.AddOrderRow(ur, obento, (DateTime)orderDate);
            }
            else             // 編集モード
            {
                ObentoDataSet.OrderRow row = (ObentoDataSet.OrderRow)((DataRowView)myCVS.View.CurrentItem).Row;
                row.Obento_ID = obento.ID;
                row.Date = (DateTime)orderDate;
            }
            // Obentoテーブルアダプタで、Obentoデータをデータベースに書き込む
            try
            {
                orderTA.Update(myDS.Order);
            }
            catch (DBConcurrencyException de)
            {
                MessageBox.Show("登録に失敗しました。\n" + de.Message);
                return;
            }
            catch (Exception ex)
            {
                MessageBox.Show("エラーが発生しました。\n" + ex.Message);
                return;
            }

            // コレクションビューソースを更新
            myCVS.View.Refresh();
            MessageBox.Show("注文を受け付けました。");

            if (addMode) // 新規登録モード
            {
                // コレクションビューソース内の新規行に移動
                InitViewSource();
            }
            else              // 編集モード
            {
                if (myCVS.View.CurrentPosition < ((CollectionView)myCVS.View).Count - 1)
                {
                    myCVS.View.MoveCurrentToNext();
                    ChangeViewSource();
                }
                else
                {
                    InitViewSource();
                }
            }
            ChangeButtonState();

        }

        private void ObentoComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // お弁当を取得
            object comboSel = ObentoComboBox.SelectedValue;
            if (comboSel == null)
            {
                TextBlockDescription.Text = "";
                PhotoImage.Source = null;
            } else {
                ObentoDataSet.ObentoRow obento = (ObentoDataSet.ObentoRow)((DataRowView)comboSel).Row;
                TextBlockDescription.Text = obento.Description;
                ImageSourceConverter imgSrcConv = new ImageSourceConverter();
                if (obento.Photo != null)
                {
                    PhotoImage.Source = (ImageSource)imgSrcConv.ConvertFrom(obento.Photo);
                }
            }
        }

        #endregion

        #region リボン・ボタンのClickイベント・ハンドラ

        private void ButtonAddOrderEntry_Click(object sender, RoutedEventArgs e)
        {
            InitViewSource();
            ChangeButtonState();
        }

        private void ButtonFirstOrderEntry_Click(object sender, RoutedEventArgs e)
        {
            myCVS.View.MoveCurrentToFirst();
            ChangeViewSource();
            ChangeButtonState();
        }

        private void ButtonPrevOrderEntry_Click(object sender, RoutedEventArgs e)
        {
            if (myCVS.View.CurrentPosition == -1)
            {
                // 新規登録モードから編集モードへ変更された場合
                myCVS.View.MoveCurrentToLast();
            }
            else if (myCVS.View.CurrentPosition > 0)
            {
                myCVS.View.MoveCurrentToPrevious();
            }
            ChangeViewSource(); 
            ChangeButtonState();
        }

        private void ButtonNextOrderEntry_Click(object sender, RoutedEventArgs e)
        {
            if (myCVS.View.CurrentPosition == -1)
            {
                // 新規登録モードから編集モードへ変更された場合
                myCVS.View.MoveCurrentToLast();
            }
            else if (myCVS.View.CurrentPosition < ((CollectionView)myCVS.View).Count - 1)
            {
                myCVS.View.MoveCurrentToNext();
            }
            ChangeViewSource();
            ChangeButtonState();
        }

        private void ButtonLastOrderEntry_Click(object sender, RoutedEventArgs e)
        {
            myCVS.View.MoveCurrentToLast();
            ChangeViewSource();
            ChangeButtonState();
        }

        private void ButtonDelOrderEntry_Click(object sender, RoutedEventArgs e)
        {
            e.Handled = true;

            if (MessageBox.Show("現在のデータを1件、削除します。OKでしょうか？",
                "確認", MessageBoxButton.OKCancel, MessageBoxImage.Question,
                MessageBoxResult.Cancel) == MessageBoxResult.Cancel)
                return;

            if (myCVS.View.CurrentPosition == -1)
            {
                return;
            }

            // 現在のOrderデータ行を取得
            ((DataRowView)myCVS.View.CurrentItem).Row.Delete();

            // Orderテーブルアダプタで、データをデータベースに書き込む
            try
            {
                orderTA.Update(myDS.Order);
            }
            catch (DBConcurrencyException)
            {
                MessageBox.Show("削除に失敗しました。すでに誰かによって削除されたようです。\n念のため、いったん全データを読み直します。");
                if (main.SelectedUser != null)
                {
                    try
                    {
                        orderTA.FillBy(myDS.Order, main.SelectedUser.ID);
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show("注文データの読み込みでエラーが発生しました。\n" + ex.Message);
                        return;
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("エラーが発生しました。\n" + ex.Message);
                return;
            }

            // コレクションビューを更新
            myCVS.View.Refresh();

            if (myCVS.View.CurrentPosition == -1)
            {
                InitViewSource();
            }
            else
            {
                ChangeViewSource();
            }

            ChangeButtonState();
        }

        private void ChangeButtonState()
        {
            // 新規登録モードの場合
            ButtonOrder.Content = "注文";
            if (myCVS.View.CurrentPosition == -1)
            {
                this.Background = this.FindResource("BackgroundAddMode") as System.Windows.Media.Brush;
                bool existOrder = (((CollectionView)myCVS.View).Count > 0);
                main.ButtonFirstOrderEntry.IsEnabled = existOrder;
                main.ButtonPrevOrderEntry.IsEnabled = existOrder;
                main.ButtonNextOrderEntry.IsEnabled = existOrder;
                main.ButtonLastOrderEntry.IsEnabled = existOrder;
                main.ButtonAddOrderEntry.IsEnabled = false;
                main.ButtonDelOrderEntry.IsEnabled = false;
                return;
            }

            // 編集モードの場合
            this.Background = this.FindResource("BackgroundEditMode") as System.Windows.Media.Brush;
            ButtonOrder.Content = "変更";
            main.ButtonAddOrderEntry.IsEnabled = true;
            main.ButtonDelOrderEntry.IsEnabled = true;
            bool existPrev = (myCVS.View.CurrentPosition > 0);
            main.ButtonFirstOrderEntry.IsEnabled = existPrev;
            main.ButtonPrevOrderEntry.IsEnabled = existPrev;
            int count = ((CollectionView)myCVS.View).Count;
            bool existNext = (myCVS.View.CurrentPosition < count - 1);
            main.ButtonNextOrderEntry.IsEnabled = existNext;
            main.ButtonLastOrderEntry.IsEnabled = existNext;
        }

        #endregion

        private void InitViewSource()
        {
            myCVS.View.MoveCurrentTo(null);
            OrderDatePicker.Text = DateTime.Today.ToShortDateString();
            ObentoComboBox.SelectedIndex = -1;
        }

        private void ChangeViewSource()
        {
            ObentoDataSet.OrderRow row = (ObentoDataSet.OrderRow)((DataRowView)myCVS.View.CurrentItem).Row;
            OrderDatePicker.Text = row.Date.ToShortDateString();

            ObentoDataSet.ObentoRow tempObento;
            int indexSelect = -1;
            for (int i = 0; i < ObentoComboBox.Items.Count; i++)
            {
                tempObento = (ObentoDataSet.ObentoRow)((DataRowView)ObentoComboBox.Items[i]).Row;
                if (tempObento.ID == row.ObentoRow.ID)
                {
                    indexSelect = i;
                    break;
                }
            }
            if (indexSelect != -1)
                ObentoComboBox.SelectedIndex = indexSelect;        
        }

        void UserSelection_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // Orderテーブルアダプタで、データセットにObentoデータを読み込む
            if ((main.SelectedUser != null) && (myDS != null))
            {
                try
                {
                    orderTA.FillBy(myDS.Order, main.SelectedUser.ID);
                }
                catch (Exception ex)
                {
                    MessageBox.Show("注文データの読み込みでエラーが発生しました。\n" + ex.Message);
                    return;
                }
                InitViewSource();
                ChangeButtonState();
            }
        }

    }
}
