﻿using System.Windows;
using System.Windows.Controls;
using ObentoApp.ObentoDataSetTableAdapters;
using System;
using System.ComponentModel;

namespace ObentoApp
{
    /// <summary>
    /// ContetOrderList.xaml の相互作用ロジック
    /// </summary>
    public partial class ContetOrderList : UserControl
    {
        private MainWindow main;            // メイン・ウィンドウ
        private ObentoDataSet myDS;         // データセット
        private OrderListTableAdapter myTA;      // OrderListテーブルアダプタ
        //private CollectionViewSource myCVS; // コレクションビューソース

        public ContetOrderList()
        {
            InitializeComponent();

            // MainWindowを取得して、各リボン・ボタンのイベント・ハンドラを追加
            main = (MainWindow)Application.Current.MainWindow;
            main.ButtonPrintOrderList.Click += new RoutedEventHandler(ButtonPrintOrderList_Click);
        }

        void ButtonPrintOrderList_Click(object sender, RoutedEventArgs e)
        {
            e.Handled = true;

            DateTime? orderDate = OrderDatePicker.SelectedDate;
            if (orderDate.HasValue == false)
            {
                MessageBox.Show("日付けが選択されていません。");
                return;
            }

            // 印刷ダイアログを表示
            PrintDialog dialog = new PrintDialog();
            dialog.PageRangeSelection = PageRangeSelection.AllPages;
            dialog.UserPageRangeEnabled = true;
            bool print = dialog.ShowDialog() ?? false;
            if (print)
            {
                // ［お弁当一覧］コンテンツ内のGridパネルを印刷
                dialog.PrintVisual(ContentGridPanel, 
                    ((DateTime)orderDate).ToShortDateString() + "の注文");
            }
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            OrderDatePicker.Text = DateTime.Today.ToShortDateString();

            // デザイン時にはデータは読み込まれません
            if (!DesignerProperties.GetIsInDesignMode(this))
            {
                // XAMLファイル側で作成されたデータセットを取得
                myDS = ((ObentoDataSet)(this.FindResource("obentoDataSet")));

                // OrderListテーブルアダプタを生成
                myTA = new OrderListTableAdapter();
                ChangeDataSource();

                // XAMLファイル側で作成されたコレクションビューソースを取得
                //myCVS = (CollectionViewSource)this.Resources["orderListViewSource"];
            }
        }

        private void OrderDatePicker_SelectedDateChanged(object sender, SelectionChangedEventArgs e)
        {
            ChangeDataSource();
        }

        private void ChangeDataSource()
        {
            DateTime? orderDate = OrderDatePicker.SelectedDate;
            if ((myDS != null) && (orderDate.HasValue))
            {
                // OrderListテーブルアダプタで、データセットにOrderListデータを読み込む
                try
                {
                    myTA.Fill(myDS.OrderList, (DateTime)orderDate);
                }
                catch (Exception ex)
                {
                    MessageBox.Show("注文データの読み込みでエラーが発生しました。\n" + ex.Message);
                    return;
                }
            }
        }

    }
}
