﻿using System;
using System.ComponentModel;
using System.Data;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using ObentoApp.ObentoDataSetTableAdapters;

namespace ObentoApp
{
    /// <summary>
    /// ContetUserEntry.xaml の相互作用ロジック
    /// </summary>
    public partial class ContetUserEntry : UserControl
    {
        private MainWindow main;            // メイン・ウィンドウ
        private CollectionViewSource myCVS; // コレクションビューソース

        public ContetUserEntry()
        {
            InitializeComponent();

            // MainWindowを取得して、各リボン・ボタンのイベント・ハンドラを追加
            main = (MainWindow)Application.Current.MainWindow;
            main.ButtonAddUserEntry.Click +=  new RoutedEventHandler(ButtonAddUserEntry_Click);
            main.ButtonFirstUserEntry.Click += new RoutedEventHandler(ButtonFirstUserEntry_Click);
            main.ButtonPrevUserEntry.Click += new RoutedEventHandler(ButtonPrevUserEntry_Click);
            main.ButtonNextUserEntry.Click += new RoutedEventHandler(ButtonNextUserEntry_Click);
            main.ButtonLastUserEntry.Click += new RoutedEventHandler(ButtonLastUserEntry_Click);
            main.ButtonDelUserEntry.Click += new RoutedEventHandler(ButtonDelUserEntry_Click);
        }

        #region データの初期ロードと、［登録］ボタンのClickイベント・ハンドラ
        
        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            // デザイン時にはデータは読み込まれません
            if (!DesignerProperties.GetIsInDesignMode(this))
            { 
                // XAMLファイル側で作成されたコレクションビューソースを取得
                myCVS = (CollectionViewSource)this.Resources["userViewSource"];
                // コレクションビューソース内の新規行に移動
                myCVS.View.MoveCurrentTo(null);

                ChangeButtonState();
            }
        }

        private void ButtonAdd_Click(object sender, RoutedEventArgs e)
        {
            // ユーザー名を取得
            string name = nameTextBox.Text.Trim();
            if (String.IsNullOrEmpty(name))
            {
                MessageBox.Show("ユーザー名を入力してください。");
                return;
            }
            // お弁当当番かどうかを取得（Nullable型の判定で??演算子を使用）
            bool dutyPerson = dutyPersonCheckBox.IsChecked ?? false;

            bool addMode = (myCVS.View.CurrentPosition == -1);
            if (addMode)
            {   // 新規登録モード
                // データセット内のUserデータテーブルに新規行を追加
                main.ObentoDS.User.AddUserRow(name, dutyPerson);
            }
            else
            {   // 編集モード
                ObentoDataSet.UserRow row = ((DataRowView)myCVS.View.CurrentItem).Row as ObentoDataSet.UserRow;
                row.Name = name;
                row.DutyPerson = dutyPerson;
            }
            //  Userテーブルアダプタで、Userデータをデータベースに書き込む
            try
            {
                main.UserTA.Update(main.ObentoDS.User);
            }
            catch (DBConcurrencyException de)
            {
                MessageBox.Show("登録に失敗しました。\n" + de.Message);
                return;
            }
            catch (Exception ex)
            {
                MessageBox.Show("エラーが発生しました。\n" + ex.Message);
                return;
            }

            // コレクションビューを更新
            myCVS.View.Refresh();

            if (addMode)
            {   // 新規登録モード
                // コレクションビューソース内の新規行に移動
                myCVS.View.MoveCurrentTo(null);
            }
            else
            {   // 編集モード
                if (myCVS.View.CurrentPosition < ((CollectionView)myCVS.View).Count - 1)
                {
                    myCVS.View.MoveCurrentToNext();
                }
                else
                {
                    myCVS.View.MoveCurrentTo(null);
                }
            }
            ChangeButtonState();
        }

        #endregion

        #region リボン・ボタンのClickイベント・ハンドラ

        private void ButtonAddUserEntry_Click(object sender, RoutedEventArgs e)
        {
            myCVS.View.MoveCurrentTo(null);
            ChangeButtonState();
        }

        private void ButtonFirstUserEntry_Click(object sender, RoutedEventArgs e)
        {
            myCVS.View.MoveCurrentToFirst();
            ChangeButtonState();
        }

        private void ButtonPrevUserEntry_Click(object sender, RoutedEventArgs e)
        {
            if (myCVS.View.CurrentPosition == -1)
            {
                // 新規登録モードから編集モードへ変更された場合
                myCVS.View.MoveCurrentToLast();
            }
            else if (myCVS.View.CurrentPosition > 0)
            {
                myCVS.View.MoveCurrentToPrevious();
            }
            ChangeButtonState();
        }

        private void ButtonNextUserEntry_Click(object sender, RoutedEventArgs e)
        {
            if (myCVS.View.CurrentPosition == -1)
            {
                // 新規登録モードから編集モードへ変更された場合
                myCVS.View.MoveCurrentToLast();
            }
            else if (myCVS.View.CurrentPosition < ((CollectionView)myCVS.View).Count - 1)
            {
                myCVS.View.MoveCurrentToNext();
            }
            ChangeButtonState();
        }

        private void ButtonLastUserEntry_Click(object sender, RoutedEventArgs e)
        {
            myCVS.View.MoveCurrentToLast();
            ChangeButtonState();
        }

        private void ButtonDelUserEntry_Click(object sender, RoutedEventArgs e)
        {
            e.Handled = true;

            if (MessageBox.Show("現在のデータを1件、削除します。OKでしょうか？",
                "確認", MessageBoxButton.OKCancel, MessageBoxImage.Question,
                MessageBoxResult.Cancel) == MessageBoxResult.Cancel)
                return;

            if (myCVS.View.CurrentPosition == -1)
            {
                return;
            }

            // 現在のUserデータ行を取得
            ((DataRowView)myCVS.View.CurrentItem).Row.Delete();

            // Userテーブルアダプタで、データをデータベースに書き込む
            try
            {
                // 先にOrderテーブル内の関連データを削除
                DataRow[] rows = main.ObentoDS.Order.Select("", "", DataViewRowState.Deleted);
                if (rows.Length > 0)
                {
                    main.OrderTA.Update(rows);
                }
                // 続いてUserテーブル内の該当データを削除
                main.UserTA.Update(main.ObentoDS.User);
            }
            catch (DBConcurrencyException de)
            {
                MessageBox.Show("削除に失敗しました。すでに誰かによって削除されたようです。\n" + de.Message);
                return;
            }
            catch (Exception ex)
            {
                MessageBox.Show("エラーが発生しました。\n" + ex.Message);
                return;
            }

            // コレクションビューを更新
            myCVS.View.Refresh();

            ChangeButtonState();
        }

        private void ChangeButtonState()
        {
            // 新規登録モードの場合
            ButtonAdd.Content = "登録";
            if (myCVS.View.CurrentPosition == -1)
            {
                this.Background = this.FindResource("BackgroundAddMode") as System.Windows.Media.Brush;
                bool existUser = (((CollectionView)myCVS.View).Count > 0);
                main.ButtonFirstUserEntry.IsEnabled = existUser;
                main.ButtonPrevUserEntry.IsEnabled = existUser;
                main.ButtonNextUserEntry.IsEnabled = existUser;
                main.ButtonLastUserEntry.IsEnabled = existUser;
                main.ButtonAddUserEntry.IsEnabled = false;
                main.ButtonDelUserEntry.IsEnabled = false;
                return;
            }

            // 編集モードの場合
            this.Background = this.FindResource("BackgroundEditMode") as System.Windows.Media.Brush;
            ButtonAdd.Content = "更新";
            main.ButtonAddUserEntry.IsEnabled = true;
            main.ButtonDelUserEntry.IsEnabled = true;
            bool existPrev = (myCVS.View.CurrentPosition > 0);
            main.ButtonFirstUserEntry.IsEnabled = existPrev;
            main.ButtonPrevUserEntry.IsEnabled = existPrev;
            int count = ((CollectionView)myCVS.View).Count;
            bool existNext = (myCVS.View.CurrentPosition < count - 1);
            main.ButtonNextUserEntry.IsEnabled = existNext;
            main.ButtonLastUserEntry.IsEnabled = existNext;
        }

        #endregion
    }
}
