﻿using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using ObentoApp.ObentoDataSetTableAdapters;
using System;
using System.Data;

namespace ObentoApp
{
    /// <summary>
    /// ContetUserList.xaml の相互作用ロジック
    /// </summary>
    public partial class ContetUserList : UserControl
    {
        private MainWindow main;            // メイン・ウィンドウ
        //private CollectionViewSource myCVS; // コレクションビューソース

        public ContetUserList()
        {
            InitializeComponent();

            // MainWindowを取得して、各リボン・ボタンのイベント・ハンドラを追加
            main = (MainWindow)Application.Current.MainWindow;
            main.ButtonSaveUserList.Click += new RoutedEventHandler(ButtonSaveUserList_Click);
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            // デザイン時にはデータは読み込まれません
            if (!DesignerProperties.GetIsInDesignMode(this))
            {
                // XAMLファイル側で作成されたコレクションビューソースを取得
                //myCVS = (CollectionViewSource)this.Resources["userViewSource"];
            }
        }

        private void UserControl_Unloaded(object sender, RoutedEventArgs e)
        {
            SaveData();
        }

        private void userDataGrid_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (userDataGrid.SelectedIndex >= userDataGrid.Items.Count - 1)
            {   // 新規登録モード
                this.Background = Application.Current.Resources["BackgroundAddMode"] as System.Windows.Media.Brush;
            }
            else
            {   // 編集モード
                this.Background = Application.Current.Resources["BackgroundEditMode"] as System.Windows.Media.Brush;
            }
        }

        private void ButtonSaveUserList_Click(object sender, RoutedEventArgs e)
        {
            e.Handled = true;
            SaveData();
        }

        private void SaveData()
        {
            //  Userテーブルアダプタで、Userデータをデータベースに書き込む
            try
            {
                // 先にOrderテーブル内の関連データを削除
                DataRow[] rows = main.ObentoDS.Order.Select("", "", DataViewRowState.Deleted);
                if (rows.Length > 0)
                {
                    main.OrderTA.Update(rows);
                }
                // 続いてUserテーブル内の該当データを削除／更新
                main.UserTA.Update(main.ObentoDS.User);
            }
            catch (DBConcurrencyException de)
            {
                MessageBox.Show("登録／削除に失敗しました。\n" + de.Message);
                return;
            }
            catch (Exception ex)
            {
                MessageBox.Show("エラーが発生しました。\n" + ex.Message);
                return;
            }
        }

    }
}
