﻿using System;
using System.Data;
using System.Reflection;
using System.Windows;
using System.Windows.Controls.Primitives;
using Microsoft.Windows.Controls.Ribbon;
using ObentoApp.ObentoDataSetTableAdapters;

namespace ObentoApp
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : RibbonWindow
    {
        public ObentoDataSet ObentoDS;          // データセット
        public UserTableAdapter UserTA;         // Userテーブルアダプタ
        public ObentoTableAdapter ObentoTA;     // Obentoテーブルアダプタ
        public OrderTableAdapter OrderTA;       // Orderテーブルアダプタ

        public ObentoDataSet.UserRow SelectedUser { get; set; }

        public MainWindow()
        {
            InitializeComponent();

            ContentControlMain.Content = new ContetObentoOrder();

            Assembly mainAssembly = Assembly.GetEntryAssembly();
            AssemblyName mainAssemName = mainAssembly.GetName();
            this.Title = String.Format("{0} {1}", this.Title, mainAssemName.Version.ToString());

            MenuItemDataReload.Click += new RoutedEventHandler(MenuItemDataReload_Click);
        }

        private void RibbonWindow_Loaded(object sender, RoutedEventArgs e)
        {
            ObentoDS = ((ObentoDataSet)(this.FindResource("CommonDataSet")));
            OrderTA = new OrderTableAdapter();
            UserTA = new UserTableAdapter();
            ObentoTA = new ObentoTableAdapter();
            ReloadOrderData();
            ReloadUserData();
            ReloadObentoData();
        }

        public bool ReloadOrderData()
        {
            // Obentoテーブルアダプタで、データセットにObentoデータを読み込む
            try
            {
                ObentoTA.Fill(ObentoDS.Obento);
            }
            catch (Exception ex)
            {
                MessageBox.Show("お弁当データの読み込みでエラーが発生しました。\n" + ex.Message);
                return false;
            }
            return true;
        }

        public bool ReloadObentoData()
        {
            // Orderテーブルアダプタで、データセットにOrderデータを読み込む
            try
            {
                OrderTA.Fill(ObentoDS.Order);
            }
            catch (Exception ex)
            {
                MessageBox.Show("注文データの読み込みでエラーが発生しました。\n" + ex.Message);
                return false;
            }
            return true;
        }

        public bool ReloadUserData()
        {
            // Userテーブルアダプタで、データセットにUserデータを読み込む
            try
            {
                UserTA.Fill(ObentoDS.User);
            }
            catch (Exception ex)
            {
                MessageBox.Show("ユーザー・データの読み込みでエラーが発生しました。\n" + ex.Message);
                return false;
            }

            UserSelection.ItemsSource = ObentoDS.User;
            UserSelection.DisplayMemberPath = "Name";

            // 前回選択されていたユーザーの選択
            string name = ((App)Application.Current).CurrentUser;
            if (String.IsNullOrWhiteSpace(name))
            {
                return true;
            }
            int indexSelect = -1;
            for (int i = 0; i < UserSelection.Items.Count; i++)
            {
                SelectedUser = ((ObentoDataSet.UserRow)((DataRowView)UserSelection.Items[i]).Row);
                if (SelectedUser.Name.Trim() == name)
                {
                    indexSelect = i;
                    break;
                }
            }
            if (indexSelect != -1)
            {
                UserSelection.SelectedIndex = indexSelect;
            }

            return true;
        }

        #region メニューの切り替え処理

        private void Menu_Checked(object sender, RoutedEventArgs e)
        {
            if (ContentControlMain != null)
            {
                SwitchContent(((ToggleButton)sender).Name);
            }
        }

        private void SwitchContent(string senderName)
        {
            if (senderName != "MenuObentoOrder") 
                MenuObentoOrder.IsChecked = false;
            if (senderName != "MenuOrderList")
                MenuOrderList.IsChecked = false; 
            if (senderName != "MenuObentoEntry") 
                MenuObentoEntry.IsChecked = false;
            if (senderName != "MenuObentoList") 
                MenuObentoList.IsChecked = false;
            if (senderName != "MenuUserEntry") 
                MenuUserEntry.IsChecked = false;
            if (senderName != "MenuUserList")
                MenuUserList.IsChecked = false;

            switch (senderName)
            {
                case "MenuObentoOrder":
                    ContentControlMain.Content = new ContetObentoOrder();
                    ChangeRibbonTabVisibility(senderName);
                    break;
                case "MenuOrderList":
                    ContentControlMain.Content = new ContetOrderList();
                    ChangeRibbonTabVisibility(senderName);
                    break;
                case "MenuObentoEntry":
                    ContentControlMain.Content = new ContetObentoEntry();
                    ChangeRibbonTabVisibility(senderName);
                    break;
                case "MenuObentoList":
                    ContentControlMain.Content = new ContetObentoList();
                    ChangeRibbonTabVisibility(senderName);
                    break;
                case "MenuUserEntry":
                    ContentControlMain.Content = new ContetUserEntry();
                    ChangeRibbonTabVisibility(senderName);
                    break;
                case "MenuUserList":
                    ContentControlMain.Content = new ContetUserList();
                    ChangeRibbonTabVisibility(senderName);
                    break;
            }
        }

        private void ChangeRibbonTabVisibility(string senderName)
        {
            if (senderName == "MenuObentoOrder") 
                TabObentoOrder.Visibility = Visibility.Visible;
            if (senderName == "MenuOrderList")
                TabOrderList.Visibility = Visibility.Visible;
            if (senderName == "MenuObentoEntry") 
                TabObentoEntry.Visibility = Visibility.Visible;
            if (senderName == "MenuObentoList")
                TabObentoList.Visibility = Visibility.Visible;
            if (senderName == "MenuUserEntry") 
                TabUserEntry.Visibility = Visibility.Visible;
            if (senderName == "MenuUserList") 
                TabUserList.Visibility = Visibility.Visible;

            if (senderName != "MenuObentoOrder") 
                TabObentoOrder.Visibility = Visibility.Hidden;
            if (senderName != "MenuOrderList")
                TabOrderList.Visibility = Visibility.Hidden;
            if (senderName != "MenuObentoEntry") 
                TabObentoEntry.Visibility = Visibility.Hidden;
            if (senderName != "MenuObentoList") 
                TabObentoList.Visibility = Visibility.Hidden;
            if (senderName != "MenuUserEntry") 
                TabUserEntry.Visibility = Visibility.Hidden;
            if (senderName != "MenuUserList") 
                TabUserList.Visibility = Visibility.Hidden;
        }


        private void Menu_Unchecked(object sender, RoutedEventArgs e)
        {
            if (ContentControlMain != null)
            {
                if ((MenuObentoOrder.IsChecked == false) &&
                    (MenuOrderList.IsChecked == false) &&
                    (MenuObentoEntry.IsChecked == false) &&
                    (MenuObentoList.IsChecked == false) &&
                    (MenuUserEntry.IsChecked == false) &&
                    (MenuUserList.IsChecked == false))
                {
                    ((ToggleButton)sender).IsChecked = true;
                }
            }
        }

        #endregion

        #region ユーザー選択処理

        private void UserSelection_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            DataRowView drv = (DataRowView)UserSelection.SelectedItem;
            if (drv == null) return;
            SelectedUser = (ObentoDataSet.UserRow)drv.Row;

            string name = SelectedUser.Name.Trim();
            UserName.Text = name + " さん";
            ((App)Application.Current).SaveUserSetting(name);

            ExpanderAdmin.IsExpanded = SelectedUser.DutyPerson;
        }

        private void MenuItemDataReload_Click(object sender, RoutedEventArgs e)
        {
            e.Handled = true;
            ObentoDS.Clear();
            ReloadOrderData();
            ReloadUserData();
            ReloadObentoData();
        }
        
        #endregion

    }
}
